@@ -137,14 +137,15 @@ static pid_t
 read_pid(const char *pidfile)
 {
 	FILE *fp;
-	pid_t pid = 0;
+	pid_t pid;
 
 	if ((fp = fopen(pidfile, "r")) == NULL) {
 		errno = ENOENT;
 		return 0;
 	}
 
-	fscanf(fp, "%d", &pid);
+	if (fscanf(fp, "%d", &pid) != 1)
+		pid = 0;
 	fclose(fp);
 
 	return pid;
@@ -316,7 +317,7 @@ parse_option(int opt, char *oarg, struct options *options)
 		break;
 	case 'h':
 		if (oarg)
-			s = parse_string(options->hostname + 1,
+			s = parse_string(options->hostname,
 					 HOSTNAME_MAX_LEN, oarg);
 		else
 			s = 0;
@@ -324,11 +325,11 @@ parse_option(int opt, char *oarg, struct options *options)
 			logger(LOG_ERR, "hostname: %s", strerror(errno));
 			return -1;
 		}
-		if (s != 0 && options->hostname[1] == '.') {
+		if (s != 0 && options->hostname[0] == '.') {
 			logger(LOG_ERR, "hostname cannot begin with a .");
 			return -1;
 		}
-		options->hostname[0] = (uint8_t)s;
+		options->hostname[s] = '\0';
 		break;
 	case 'i':
 		if (oarg)
@@ -648,11 +649,12 @@ main(int argc, char **argv)
 	}
 #endif
 
-	gethostname(options->hostname + 1, sizeof(options->hostname));
-	if (strcmp(options->hostname + 1, "(none)") == 0 ||
-	    strcmp(options->hostname + 1, "localhost") == 0)
-		options->hostname[1] = '\0';
-	*options->hostname = strlen(options->hostname + 1);
+	gethostname(options->hostname, HOSTNAME_MAX_LEN);
+	/* Ensure that the hostname is NULL terminated */ 
+	options->hostname[HOSTNAME_MAX_LEN] = '\0';
+	if (strcmp(options->hostname, "(none)") == 0 ||
+	    strcmp(options->hostname, "localhost") == 0)
+		options->hostname[0] = '\0';
 
 	while ((opt = getopt_long(argc, argv, OPTS EXTRA_OPTS,
 				  longopts, &option_index)) != -1)
@@ -877,7 +879,8 @@ main(int argc, char **argv)
 		goto abort;
 	}
 
-	chdir("/");
+	if (chdir("/") == -1)
+		logger(LOG_ERR, "chdir `/': %s", strerror(errno));
 	umask(022);
 
 	if (sig != 0 && !(options->options & DHCPCD_DAEMONISED)) {
